import random

CAR = "c"
GOAT = "g"
PICK = "*"
UNKNOWN = "?"


def generateDoors(n):
    """generate a random list of door contents with 1 CAR and n-1 GOATs"""

    # start with 1 car
    doors = [CAR]

    # add `n-1` goats
    for i in range(n - 1):
        doors.append(GOAT)

    # shuffle the result
    random.shuffle(doors)

    # return it
    return doors


def initializeGame(n):
    """create the initial game state, all UNKNOWN"""
    return [UNKNOWN] * n


def printDoors(game_state):
    """Print out the game state."""

    # we will construct a space-separated string of our game state
    print("\nGame: {}\n".format(" ".join(game_state)))


def revealGoats(game_state, door_contents):
    """randomly reveal len(doors-1) goats"""

    # leave 1 remaining goat, and omit the user's pick, so let to_open be the length of the contents - 2
    to_open = len(door_contents) - 2

    # we will open doors until count == to_open, so initialize count to 0
    count = 0

    # open doors until count == to_open
    while count < to_open:
        # pick a random door
        try_to_open = random.randint(0, len(door_contents) - 1)

        # if its not already opened, its not the user's pick, and its not the car...
        if (
            game_state[try_to_open] != GOAT
            and game_state[try_to_open] != PICK
            and door_contents[try_to_open] != CAR
        ):
            # open it and increment `count`
            game_state[try_to_open] = GOAT
            count += 1
    return game_state


def revealGoatsSequential(game_state, door_contents):
    """reveal len(doors-1) goats"""

    # leave 1 remaining goat, and omit the user's pick, so let to_open be the length of the contents - 2
    to_open = len(door_contents) - 2

    # we will open doors until count == to_open, so initialize count to 0
    count = 0

    # open doors until count == to_open
    for i in range(len(game_state)):
        if door_contents[i] == CAR:
            # don't want to reveal a car!
            continue
        if game_state[i] == PICK:
            # don't reveal the user's pick
            continue
        if count >= to_open:
            # we've revealed enough
            break

        # reveal the door
        game_state[i] = door_contents[i]
        count += 1

    return game_state


def switchUserPick(game_state):
    """Get the user's pick *, and the remaining unopened door ?, and switch them"""
    user_pick = game_state.index(PICK)
    unopened_door = game_state.index(UNKNOWN)
    game_state[user_pick] = UNKNOWN
    game_state[unopened_door] = PICK
    return game_state


def gameResult(game_state, door_contents):
    user_pick = game_state.index(PICK)
    contents = door_contents[user_pick]
    return contents == CAR


def playSingleGame(num_doors):
    play_again = "y"
    while play_again == "y":
        # setup the game, `doors` is the list of door contents either CAR or GOAT
        door_contents = generateDoors(num_doors)

        # initialize the game state, starts as all UNKNOWN
        game_state = initializeGame(len(door_contents))

        # print the doors
        printDoors(game_state)

        # ask the user to pick a door
        user_pick = int(
            input("Please pick a door between 0 and {}: ".format(num_doors - 1))
        )

        # update game state with the users pick, changing from UNKNOWN to PICK
        game_state[user_pick] = PICK

        # print the doors
        printDoors(game_state)

        # reveal the doors' contents
        print("Revealing the goats!")
        game_state = revealGoats(game_state, door_contents)

        # print the doors
        printDoors(game_state)

        # ask if they want to switch to the only remaining unknown door
        switch = input("Would you like to switch (y/n): ")

        # switch if they asked to do so
        if switch == "y":
            game_state = switchUserPick(game_state)

        # resolve the game
        print("You picked door {}".format(game_state.index(PICK)))
        print("The door contents were:")
        printDoors(door_contents)

        # result is true if we won, else false
        result = gameResult(game_state, door_contents)

        if result:
            print("You won the car!\n")
        else:
            print("You lose!\n")

        # ask if they want to play again
        play_again = input("Would you like to play again (y/n): ")


def simulateSingleGame(num_doors, num_simulations):
    """Here, we will simulate playing the game with `num_doors` doors, `num_simulations` times

    In this simulation, we will *always choose to switch*.
    """
    num_wins = 0
    num_losses = 0

    for i in range(num_simulations):
        # initialize game
        door_contents = generateDoors(num_doors)

        # initialize the game state
        game_state = initializeGame(len(door_contents))

        # generate a pick
        user_pick = random.randint(0, num_doors - 1)

        # update game state
        game_state[user_pick] = PICK

        # reveal the doors
        game_state = revealGoats(game_state, door_contents)

        # switch the pick
        game_state = switchUserPick(game_state)

        # resolve the game
        result = gameResult(game_state, door_contents)

        if result:
            num_wins += 1
        else:
            num_losses += 1
    return num_wins, num_losses


def main():
    run_type = input(
        "Would you like to play a single round (a), run a simulation (b): "
    )

    # ask for number of doors
    num_doors = int(input("How many doors would you like to play with: "))

    if run_type == "a":
        # play the game manually
        playSingleGame(num_doors)

    elif run_type == "b":
        # ask for number of simulations
        num_sims = int(input("How many simulations would you like to run: "))

        wins, losses = simulateSingleGame(num_doors, num_sims)

        print("There were {} wins and {} losses".format(wins, losses))
        print(
            "So keeping the original door wins with probability {:.1%}, and switching wins with probability {:.1%}".format(
                losses / num_sims, wins / num_sims
            )
        )


main()
